<h1>{{label}}</h1>
<p>{{description}}</p>
<h3>OneToOneModel</h3>
{{#onetoonemodel}}
<p><a href="/onetoonemodels/{{id}}">{{label}}</a></p>
{{/onetoonemodel}}
<h3>ExtraModels</h3>
<ul>
{{#extramodels}}
  <li><a href="/extramodels/{{id}}">{{label}}</a></li>
{{/extramodels}}
</ul>
<p>
{{#page_config.can_change}}
  <a href="/{{id}}/edit">Edit</a>
{{/page_config.can_change}}
</p>
